package com.sun.jndi.toolkit.dir;

import javax.naming.Name;
import java.util.Locale;
import javax.naming.InvalidNameException;
import java.util.Properties;
import java.util.NoSuchElementException;
import java.util.Enumeration;
import javax.naming.CompoundName;

final class HierarchicalName extends CompoundName
{
    private int hashValue;
    private static final long serialVersionUID = -6717336834584573168L;
    
    HierarchicalName() {
        super(new Enumeration<String>() {
            @Override
            public boolean hasMoreElements() {
                return false;
            }
            
            @Override
            public String nextElement() {
                throw new NoSuchElementException();
            }
        }, HierarchicalNameParser.mySyntax);
        this.hashValue = -1;
    }
    
    HierarchicalName(Enumeration<String> comps, Properties syntax) {
        super(comps, syntax);
        this.hashValue = -1;
    }
    
    HierarchicalName(String n, Properties syntax) throws InvalidNameException {
        super(n, syntax);
        this.hashValue = -1;
    }
    
    @Override
    public int hashCode() {
        if (this.hashValue == -1) {
            String upperCase = this.toString().toUpperCase(Locale.ENGLISH);
            int i = upperCase.length();
            int n = 0;
            char[] dst = new char[i];
            upperCase.getChars(0, i, dst, 0);
            for (; i > 0; --i) {
                this.hashValue = this.hashValue * 37 + dst[n++];
            }
        }
        return this.hashValue;
    }
    
    @Override
    public Name getPrefix(int posn) {
        return new HierarchicalName(super.getPrefix(posn).getAll(), this.mySyntax);
    }
    
    @Override
    public Name getSuffix(int posn) {
        return new HierarchicalName(super.getSuffix(posn).getAll(), this.mySyntax);
    }
    
    @Override
    public Object clone() {
        return new HierarchicalName(this.getAll(), this.mySyntax);
    }
}
